@@ -17,8 +17,7 @@ DEFAULT_NODATA <- list(
     "Byte" = 255, "Int8" = -128,
     "UInt16" = 65535, "Int16" = -32767,
     "UInt32" = 4294967293, "Int32" = -2147483647,
-    "Float32" = -99999.0, "Float64" = -99999.0
-)
+    "Float32" = -99999.0, "Float64" = -99999.0)
 
 
 #' List of default DEM processing options
@@ -26,34 +25,32 @@ DEFAULT_NODATA <- list(
 #' These values are used in `dem_proc()` as the default processing options:
 #' \preformatted{
 #'     list(
-#'          hillshade =    c("-z", "1", "-s", "1", "-az", "315",
-#'                           "-alt", "45", "-alg", "Horn",
-#'                           "-combined", "-compute_edges"),
-#'          slope =        c("-s", "1", "-alg", "Horn", "-compute_edges"),
-#'          aspect =       c("-alg", "Horn", "-compute_edges"),
-#'          color_relief = character(),
-#'          TRI =          c("-alg", "Riley", "-compute_edges"),
-#'          TPI =          c("-compute_edges"),
-#'          roughness =    c("-compute_edges")
-#'          )
+#'          "hillshade" =    c("-z", "1", "-s", "1", "-az", "315",
+#'                             "-alt", "45", "-alg", "Horn",
+#'                             "-combined", "-compute_edges"),
+#'          "slope" =        c("-s", "1", "-alg", "Horn", "-compute_edges"),
+#'          "aspect" =       c("-alg", "Horn", "-compute_edges"),
+#'          "color-relief" = character(),
+#'          "TRI" =          c("-alg", "Riley", "-compute_edges"),
+#'          "TPI" =          c("-compute_edges"),
+#'          "roughness" =    c("-compute_edges"))
 #' }
 #' @seealso
 #' [dem_proc()]
 #'
-#' \url{https://gdal.org/programs/gdaldem.html} for a description of all
+#' \url{https://gdal.org/en/stable/programs/gdaldem.html} for a description of all
 #' available command-line options for each processing mode
 #' @export
 DEFAULT_DEM_PROC <- list(
-    hillshade = c("-z", "1", "-s", "1", "-az", "315",
-                  "-alt", "45", "-alg", "Horn",
-                  "-combined", "-compute_edges"),
-    slope = c("-s", "1", "-alg", "Horn", "-compute_edges"),
-    aspect = c("-alg", "Horn", "-compute_edges"),
-    color_relief = character(),
-    TRI = c("-alg", "Riley", "-compute_edges"),
-    TPI = c("-compute_edges"),
-    roughness = c("-compute_edges")
-)
+    "hillshade" = c("-z", "1", "-s", "1", "-az", "315",
+                    "-alt", "45", "-alg", "Horn",
+                    "-combined", "-compute_edges"),
+    "slope" = c("-s", "1", "-alg", "Horn", "-compute_edges"),
+    "aspect" = c("-alg", "Horn", "-compute_edges"),
+    "color-relief" = character(),
+    "TRI" = c("-alg", "Riley", "-compute_edges"),
+    "TPI" = c("-compute_edges"),
+    "roughness" = c("-compute_edges"))
 
 
 .VRT_KERNEL_TEMPLATE <-
@@ -140,6 +137,7 @@ DEFAULT_DEM_PROC <- list(
 #'   $bbox = c(xmin, ymin, xmax, ymax)
 #'   $dim = c(xsize, ysize, nbands)
 #'   $srs = <projection as WKT2 string>
+#'   $datatype = <character vector of data type name by band>
 #' }
 #' The WKT version used for the projection string can be overridden by setting
 #' the `OSR_WKT_FORMAT` configuration option. See [srs_to_wkt()] for a list of
@@ -231,13 +229,15 @@ read_ds <- function(ds, bands=NULL, xoff=0, yoff=0,
     i <- 1
     readByteAsRaw <- ds$readByteAsRaw
     if (as_raw) {
-      ds$readByteAsRaw <- TRUE
-      dtype <- ds$getDataTypeName(bands[1L])
-      if (!dtype == "Byte") {
-        warning(sprintf("'as_raw' set to 'TRUE' only affects read for band type 'Byte',  current data type: '%s'", dtype))
-      }
+        ds$readByteAsRaw <- TRUE
+        dtype <- ds$getDataTypeName(bands[1L])
+        if (!dtype == "Byte") {
+            warning(sprintf("'as_raw' set to 'TRUE' only affects read for band type 'Byte', current data type: '%s'", dtype))
+        }
     }
+    dtype <- character()
     for (b in bands) {
+        dtype <- c(dtype, ds$getDataTypeName(b))
         if (as_list) {
             r[[i]] <- ds$read(b, xoff, yoff, xsize, ysize,
                               out_xsize, out_ysize)
@@ -250,19 +250,20 @@ read_ds <- function(ds, bands=NULL, xoff=0, yoff=0,
 
     ## restore the field, note that it may have had no impact
     ds$readByteAsRaw <- readByteAsRaw
+
     gt <- ds$getGeoTransform()
     ul_xy <- .apply_geotransform(gt, xoff, yoff)
     lr_xy <- .apply_geotransform(gt, (xoff + xsize), (yoff + ysize))
     bb <- c(ul_xy[1], lr_xy[2], lr_xy[1], ul_xy[2])
-
-    # gis: a list with the raster bbox, dimensions, projection
+    # gis: a list with the bbox, dimensions, projection, nbands, datatype
     wkt_fmt_config <- get_config_option("OSR_WKT_FORMAT")
     if (wkt_fmt_config == "")
         set_config_option("OSR_WKT_FORMAT", "WKT2")
     attr(r, "gis") <- list(type = "raster",
                            bbox = bb,
                            dim = c(out_xsize, out_ysize, length(bands)),
-                           srs = ds$getProjectionRef())
+                           srs = ds$getProjectionRef(),
+                           datatype = dtype)
     set_config_option("OSR_WKT_FORMAT", wkt_fmt_config)
 
     return(r)
@@ -413,7 +414,7 @@ rasterFromRaster <- function(srcfile, dstfile, fmt=NULL, nbands=NULL,
 #' elements of the XML schema describe how the source data will be read, along
 #' with algorithms potentially applied and so forth. Documentation of the XML
 #' format for .vrt is at:
-#' \url{https://gdal.org/drivers/raster/vrt.html}.
+#' \url{https://gdal.org/en/stable/drivers/raster/vrt.html}.
 #'
 #' Since .vrt is a small plain-text file it is fast to write and requires
 #' little storage space. Read performance is not degraded for certain simple
@@ -1367,7 +1368,7 @@ combine <- function(rasterfiles, var.names=NULL, bands=NULL,
 #' @description
 #' `dem_proc()` generates DEM derivatives from an input elevation raster. This
 #' function is a wrapper for the \command{gdaldem} command-line utility.
-#' See \url{https://gdal.org/programs/gdaldem.html} for details.
+#' See \url{https://gdal.org/en/stable/programs/gdaldem.html} for details.
 #'
 #' @param mode Character. Name of the DEM processing mode. One of hillshade,
 #' slope, aspect, color-relief, TRI, TPI or roughness.
@@ -1385,7 +1386,7 @@ combine <- function(rasterfiles, var.names=NULL, bands=NULL,
 #' @note
 #' Band 1 of the source elevation raster is read by default, but this can be
 #' changed by including a `-b` command-line argument in `mode_options`.
-#' See the \href{https://gdal.org/programs/gdaldem.html}{documentation for
+#' See the \href{https://gdal.org/en/stable/programs/gdaldem.html}{documentation for
 #' `gdaldem`} for a description of all available options for each processing
 #' mode.
 #'
@@ -1426,7 +1427,7 @@ dem_proc <- function(mode,
 #' exist, otherwise it will try to append to an existing one.
 #' This function is a wrapper of `GDALPolygonize` in the GDAL Algorithms API.
 #' It provides essentially the same functionality as the `gdal_polygonize.py`
-#' command-line program (\url{https://gdal.org/programs/gdal_polygonize.html}).
+#' command-line program (\url{https://gdal.org/en/stable/programs/gdal_polygonize.html}).
 #'
 #' @details
 #' Polygon features will be created on the output layer, with polygon
@@ -1628,7 +1629,7 @@ polygonize <- function(raster_file,
 #' the band(s) of a raster dataset. Vectors are read from any GDAL
 #' OGR-supported vector format.
 #' This function is a wrapper for the \command{gdal_rasterize} command-line
-#' utility (\url{https://gdal.org/programs/gdal_rasterize.html}).
+#' utility (\url{https://gdal.org/en/stable/programs/gdal_rasterize.html}).
 #'
 #' @param src_dsn Data source name for the input vector layer (filename or
 #' connection string).
