@@ -16,7 +16,7 @@
 //' `vsi_copy_file()` is a wrapper for `VSICopyFile()` in the GDAL Common
 //' Portability Library. The GDAL VSI functions allow virtualization of disk
 //' I/O so that non file data sources can be made to appear as files.
-//' See \url{https://gdal.org/user/virtual_file_systems.html}.
+//' See \url{https://gdal.org/en/stable/user/virtual_file_systems.html}.
 //' Requires GDAL >= 3.7.
 //'
 //' @details
@@ -137,23 +137,34 @@ void vsi_curl_clear_cache(bool partial = false,
 //'
 //' `vsi_read_dir()` abstracts access to directory contents. It returns a
 //' character vector containing the names of files and directories in this
-//' directory. This function is a wrapper for `VSIReadDirEx()` in the GDAL
-//' Common Portability Library.
+//' directory. With `recursive = TRUE`, reads the list of entries in the
+//' directory and subdirectories.
+//' This function is a wrapper for `VSIReadDirEx()` and `VSIReadDirRecursive()`
+//' in the GDAL Common Portability Library.
 //'
 //' @param path Character string. The relative or absolute path of a
 //' directory to read.
 //' @param max_files Integer scalar. The maximum number of files after which to
-//' stop, or 0 for no limit (see Note).
+//' stop, or 0 for no limit (see Note). Ignored if `recursive = TRUE`.
+//' @param recursive Logical scalar. `TRUE` to read the directory and its
+//' subdirectories. Defaults to `FALSE`.
+//' @param all_files Logical scalar. If `FALSE` (the default), only the names
+//' of visible files are returned (following Unix-style visibility, that is
+//' files whose name does not start with a dot). If `TRUE`, all file names
+//' will be returned.
 //' @returns A character vector containing the names of files and directories
-//' in the directory given by `path`. An empty string (`""`) is returned if
-//' `path` does not exist.
+//' in the directory given by `path`. The listing is in alphabetical order, and
+//' does not include the special entries '.' and '..' even if they are present
+//' in the directory. An empty string (`""`) is returned if `path` does not
+//' exist.
 //'
 //' @note
 //' If `max_files` is set to a positive number, directory listing will stop
 //' after that limit has been reached. Note that to indicate truncation, at
 //' least one element more than the `max_files` limit will be returned. If the
 //' length of the returned character vector is lesser or equal to `max_files`,
-//' then no truncation occurred.
+//' then no truncation occurred. The `max_files` parameter is ignored when
+//' `recursive = TRUE`.
 //'
 //' @seealso
 //' [vsi_mkdir()], [vsi_rmdir()], [vsi_stat()], [vsi_sync()]
@@ -164,22 +175,31 @@ void vsi_curl_clear_cache(bool partial = false,
 //' vsi_read_dir(data_dir)
 // [[Rcpp::export()]]
 Rcpp::CharacterVector vsi_read_dir(Rcpp::CharacterVector path,
-                                   int max_files = 0) {
+                                   int max_files = 0,
+                                   bool recursive = false,
+                                   bool all_files = false) {
 
-    std::string path_in;
-    path_in = Rcpp::as<std::string>(check_gdal_filename(path));
+    std::string path_in = Rcpp::as<std::string>(check_gdal_filename(path));
 
-    char **papszFiles;
-    papszFiles = VSIReadDirEx(path_in.c_str(), max_files);
+    char **papszFiles = nullptr;
+    if (recursive)
+        papszFiles = VSIReadDirRecursive(path_in.c_str());
+    else
+        papszFiles = VSIReadDirEx(path_in.c_str(), max_files);
 
     int nItems = CSLCount(papszFiles);
     if (nItems > 0) {
-        Rcpp::CharacterVector files(nItems);
+        std::vector<std::string> files{};
         for (int i=0; i < nItems; ++i) {
-            files(i) = papszFiles[i];
+            if (!all_files && STARTS_WITH(papszFiles[i], "."))
+                continue;
+            if (!EQUAL(papszFiles[i], ".") && !EQUAL(papszFiles[i], "..")) {
+                files.push_back(papszFiles[i]);
+            }
+            std::sort(files.begin(), files.end());
         }
         CSLDestroy(papszFiles);
-        return files;
+        return Rcpp::wrap(files);
     }
     else {
         CSLDestroy(papszFiles);
@@ -535,7 +555,7 @@ SEXP vsi_unlink_batch(Rcpp::CharacterVector filenames) {
 //'
 //' @seealso
 //' GDAL Virtual File Systems:\cr
-//' \url{https://gdal.org/user/virtual_file_systems.html}
+//' \url{https://gdal.org/en/stable/user/virtual_file_systems.html}
 //'
 //' @examples
 //' data_dir <- system.file("extdata", package="gdalraster")
@@ -660,7 +680,7 @@ int vsi_rename(Rcpp::CharacterVector oldpath, Rcpp::CharacterVector newpath) {
 //' @seealso
 //' [vsi_get_fs_options()]
 //'
-//' \url{https://gdal.org/user/virtual_file_systems.html}
+//' \url{https://gdal.org/en/stable/user/virtual_file_systems.html}
 //'
 //' @examples
 //' vsi_get_fs_prefixes()
