@@ -1,5 +1,5 @@
 /* R interface to the GDAL C API for raster
-   https://gdal.org/api/raster_c_api.html
+   https://gdal.org/en/stable/api/raster_c_api.html
 
    Chris Toney <chris.toney at usda.gov>
    Copyright (c) 2023-2024 gdalraster authors
@@ -243,6 +243,10 @@ bool deleteDataset(Rcpp::CharacterVector filename, std::string format);
 bool renameDataset(Rcpp::CharacterVector new_filename,
                    Rcpp::CharacterVector old_filename,
                    std::string format);
+SEXP identifyDriver(Rcpp::CharacterVector filename,
+                    bool raster, bool vector,
+                    Rcpp::Nullable<Rcpp::CharacterVector> allowed_drivers,
+                    Rcpp::Nullable<Rcpp::CharacterVector> file_list);
 bool bandCopyWholeRaster(Rcpp::CharacterVector src_filename, int src_band,
                          Rcpp::CharacterVector dst_filename, int dst_band,
                          Rcpp::Nullable<Rcpp::CharacterVector> options,
